# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncChunkPagination, AsyncChunkPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.knowledge_bases import chunk_list_params
from ...types.knowledge_bases.chunk_list_response import ChunkListResponse

__all__ = ["ChunksResource", "AsyncChunksResource"]


class ChunksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ChunksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return ChunksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChunksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return ChunksResourceWithStreamingResponse(self)

    def list(
        self,
        knowledge_base_id: str,
        *,
        chunk_id: str | NotGiven = NOT_GIVEN,
        max_chunks: int | NotGiven = NOT_GIVEN,
        metadata_filters: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncChunkPagination[ChunkListResponse]:
        """
        ### Description

        Get chunks from a knowledge base using chunk IDs or a matching metadata field

            ### Details
            This API can be used to get a list of chunks from a knowledge base. Given a chunk id,             a metadata field and value, or both, matching chunks are searched for in the knowledge base             given by knowledge base id.

        Args:
          chunk_id: Optional search by chunk_id

          max_chunks: Maximum number of chunks returned by the get_chunks endpoint. Defaults to 10 and
              cannot be greater than 2000.

          metadata_filters: Optional search by metadata fields, encoded as a JSON object

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return self._get_api_list(
            f"/v4/knowledge-bases/{knowledge_base_id}/chunks",
            page=SyncChunkPagination[ChunkListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "chunk_id": chunk_id,
                        "max_chunks": max_chunks,
                        "metadata_filters": metadata_filters,
                    },
                    chunk_list_params.ChunkListParams,
                ),
            ),
            model=ChunkListResponse,
        )


class AsyncChunksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncChunksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncChunksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChunksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncChunksResourceWithStreamingResponse(self)

    def list(
        self,
        knowledge_base_id: str,
        *,
        chunk_id: str | NotGiven = NOT_GIVEN,
        max_chunks: int | NotGiven = NOT_GIVEN,
        metadata_filters: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[ChunkListResponse, AsyncChunkPagination[ChunkListResponse]]:
        """
        ### Description

        Get chunks from a knowledge base using chunk IDs or a matching metadata field

            ### Details
            This API can be used to get a list of chunks from a knowledge base. Given a chunk id,             a metadata field and value, or both, matching chunks are searched for in the knowledge base             given by knowledge base id.

        Args:
          chunk_id: Optional search by chunk_id

          max_chunks: Maximum number of chunks returned by the get_chunks endpoint. Defaults to 10 and
              cannot be greater than 2000.

          metadata_filters: Optional search by metadata fields, encoded as a JSON object

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        return self._get_api_list(
            f"/v4/knowledge-bases/{knowledge_base_id}/chunks",
            page=AsyncChunkPagination[ChunkListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "chunk_id": chunk_id,
                        "max_chunks": max_chunks,
                        "metadata_filters": metadata_filters,
                    },
                    chunk_list_params.ChunkListParams,
                ),
            ),
            model=ChunkListResponse,
        )


class ChunksResourceWithRawResponse:
    def __init__(self, chunks: ChunksResource) -> None:
        self._chunks = chunks

        self.list = to_raw_response_wrapper(
            chunks.list,
        )


class AsyncChunksResourceWithRawResponse:
    def __init__(self, chunks: AsyncChunksResource) -> None:
        self._chunks = chunks

        self.list = async_to_raw_response_wrapper(
            chunks.list,
        )


class ChunksResourceWithStreamingResponse:
    def __init__(self, chunks: ChunksResource) -> None:
        self._chunks = chunks

        self.list = to_streamed_response_wrapper(
            chunks.list,
        )


class AsyncChunksResourceWithStreamingResponse:
    def __init__(self, chunks: AsyncChunksResource) -> None:
        self._chunks = chunks

        self.list = async_to_streamed_response_wrapper(
            chunks.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .artifact_schema_generation_param import ArtifactSchemaGenerationParam
from .generation_test_case_schema_param import GenerationTestCaseSchemaParam

__all__ = ["AutogeneratedDraftTestCaseUpdateParams", "TestCaseData"]


class AutogeneratedDraftTestCaseUpdateParams(TypedDict, total=False):
    evaluation_dataset_id: Required[str]

    approved: bool
    """Boolean to track whether or not the draft test case is approved"""

    schema_type: Literal["GENERATION"]

    test_case_data: TestCaseData
    """The data for the test case in a format matching the provided schema_type"""


TestCaseData: TypeAlias = Union[ArtifactSchemaGenerationParam, GenerationTestCaseSchemaParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .history import (
    HistoryResource,
    AsyncHistoryResource,
    HistoryResourceWithRawResponse,
    AsyncHistoryResourceWithRawResponse,
    HistoryResourceWithStreamingResponse,
    AsyncHistoryResourceWithStreamingResponse,
)
from .test_case_results import (
    TestCaseResultsResource,
    AsyncTestCaseResultsResource,
    TestCaseResultsResourceWithRawResponse,
    AsyncTestCaseResultsResourceWithRawResponse,
    TestCaseResultsResourceWithStreamingResponse,
    AsyncTestCaseResultsResourceWithStreamingResponse,
)

__all__ = [
    "HistoryResource",
    "AsyncHistoryResource",
    "HistoryResourceWithRawResponse",
    "AsyncHistoryResourceWithRawResponse",
    "HistoryResourceWithStreamingResponse",
    "AsyncHistoryResourceWithStreamingResponse",
    "TestCaseResultsResource",
    "AsyncTestCaseResultsResource",
    "TestCaseResultsResourceWithRawResponse",
    "AsyncTestCaseResultsResourceWithRawResponse",
    "TestCaseResultsResourceWithStreamingResponse",
    "AsyncTestCaseResultsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["GoogleDriveDataSourceAuthConfigParam"]


class GoogleDriveDataSourceAuthConfigParam(TypedDict, total=False):
    client_email: Required[str]
    """
    Client email to use for google drive, set to override client email set in env
    vars
    """

    client_id: Required[str]
    """Client id to use for google drive, set to override client id set in env vars"""

    private_key: Required[str]
    """
    Private key to use for google drive, set to override private key set in env vars
    """

    source: Required[Literal["GoogleDrive"]]

    token_uri: Required[str]
    """Token uri to use for google drive, set to override token uri set in env vars"""

    encrypted: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal

import httpx

from ..types import question_list_params, question_create_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncPageResponse, AsyncPageResponse
from .._base_client import AsyncPaginator, make_request_options
from ..types.question import Question

__all__ = ["QuestionsResource", "AsyncQuestionsResource"]


class QuestionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> QuestionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return QuestionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> QuestionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return QuestionsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        prompt: str,
        title: str,
        type: Literal["categorical", "free_text"],
        choices: Iterable[object] | NotGiven = NOT_GIVEN,
        conditions: Iterable[object] | NotGiven = NOT_GIVEN,
        dropdown: bool | NotGiven = NOT_GIVEN,
        multi: bool | NotGiven = NOT_GIVEN,
        required: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Question:
        """
        ### Description

        Creates a question

        ### Details

        This API can be used to create a question. To use this API, review the request
        schema and pass in all fields that are required to create a question.

        Args:
          account_id: The ID of the account that owns the given entity.

          type: An enumeration.

          choices: List of choices for the question. Required for CATEGORICAL questions.

          conditions: Conditions for the question to be shown.

          dropdown: Whether the question is displayed as a dropdown in the UI.

          multi: Whether the question allows multiple answers.

          required: Whether the question is required.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/questions",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "prompt": prompt,
                    "title": title,
                    "type": type,
                    "choices": choices,
                    "conditions": conditions,
                    "dropdown": dropdown,
                    "multi": multi,
                    "required": required,
                },
                question_create_params.QuestionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Question,
        )

    def retrieve(
        self,
        question_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Question:
        """
        ### Description

        Gets the details of a question

        ### Details

        This API can be used to get information about a single question by ID. To use
        this API, pass in the `id` that was returned from your Create Question API call
        as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not question_id:
            raise ValueError(f"Expected a non-empty value for `question_id` but received {question_id!r}")
        return self._get(
            f"/v4/questions/{question_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Question,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[Question]:
        """
        ### Description

        Lists all questions accessible to the user.

        ### Details

        This API can be used to list questions. If a user has access to multiple
        accounts, all questions from all accounts the user is associated with will be
        returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/questions",
            page=SyncPageResponse[Question],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                    },
                    question_list_params.QuestionListParams,
                ),
            ),
            model=Question,
        )


class AsyncQuestionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncQuestionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncQuestionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncQuestionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncQuestionsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        prompt: str,
        title: str,
        type: Literal["categorical", "free_text"],
        choices: Iterable[object] | NotGiven = NOT_GIVEN,
        conditions: Iterable[object] | NotGiven = NOT_GIVEN,
        dropdown: bool | NotGiven = NOT_GIVEN,
        multi: bool | NotGiven = NOT_GIVEN,
        required: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Question:
        """
        ### Description

        Creates a question

        ### Details

        This API can be used to create a question. To use this API, review the request
        schema and pass in all fields that are required to create a question.

        Args:
          account_id: The ID of the account that owns the given entity.

          type: An enumeration.

          choices: List of choices for the question. Required for CATEGORICAL questions.

          conditions: Conditions for the question to be shown.

          dropdown: Whether the question is displayed as a dropdown in the UI.

          multi: Whether the question allows multiple answers.

          required: Whether the question is required.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/questions",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "prompt": prompt,
                    "title": title,
                    "type": type,
                    "choices": choices,
                    "conditions": conditions,
                    "dropdown": dropdown,
                    "multi": multi,
                    "required": required,
                },
                question_create_params.QuestionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Question,
        )

    async def retrieve(
        self,
        question_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Question:
        """
        ### Description

        Gets the details of a question

        ### Details

        This API can be used to get information about a single question by ID. To use
        this API, pass in the `id` that was returned from your Create Question API call
        as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not question_id:
            raise ValueError(f"Expected a non-empty value for `question_id` but received {question_id!r}")
        return await self._get(
            f"/v4/questions/{question_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Question,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[Question, AsyncPageResponse[Question]]:
        """
        ### Description

        Lists all questions accessible to the user.

        ### Details

        This API can be used to list questions. If a user has access to multiple
        accounts, all questions from all accounts the user is associated with will be
        returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/questions",
            page=AsyncPageResponse[Question],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "limit": limit,
                        "page": page,
                    },
                    question_list_params.QuestionListParams,
                ),
            ),
            model=Question,
        )


class QuestionsResourceWithRawResponse:
    def __init__(self, questions: QuestionsResource) -> None:
        self._questions = questions

        self.create = to_raw_response_wrapper(
            questions.create,
        )
        self.retrieve = to_raw_response_wrapper(
            questions.retrieve,
        )
        self.list = to_raw_response_wrapper(
            questions.list,
        )


class AsyncQuestionsResourceWithRawResponse:
    def __init__(self, questions: AsyncQuestionsResource) -> None:
        self._questions = questions

        self.create = async_to_raw_response_wrapper(
            questions.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            questions.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            questions.list,
        )


class QuestionsResourceWithStreamingResponse:
    def __init__(self, questions: QuestionsResource) -> None:
        self._questions = questions

        self.create = to_streamed_response_wrapper(
            questions.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            questions.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            questions.list,
        )


class AsyncQuestionsResourceWithStreamingResponse:
    def __init__(self, questions: AsyncQuestionsResource) -> None:
        self._questions = questions

        self.create = async_to_streamed_response_wrapper(
            questions.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            questions.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            questions.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, TypedDict

__all__ = ["QuestionSetCreateParams"]


class QuestionSetCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    name: Required[str]

    question_ids: Required[List[str]]
    """IDs of questions in the question set"""

    instructions: str
    """Instructions to answer questions"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .models import (
    ModelsResource,
    AsyncModelsResource,
    ModelsResourceWithRawResponse,
    AsyncModelsResourceWithRawResponse,
    ModelsResourceWithStreamingResponse,
    AsyncModelsResourceWithStreamingResponse,
)
from .embeddings import (
    EmbeddingsResource,
    AsyncEmbeddingsResource,
    EmbeddingsResourceWithRawResponse,
    AsyncEmbeddingsResourceWithRawResponse,
    EmbeddingsResourceWithStreamingResponse,
    AsyncEmbeddingsResourceWithStreamingResponse,
)
from .rerankings import (
    RerankingsResource,
    AsyncRerankingsResource,
    RerankingsResourceWithRawResponse,
    AsyncRerankingsResourceWithRawResponse,
    RerankingsResourceWithStreamingResponse,
    AsyncRerankingsResourceWithStreamingResponse,
)
from .completions import (
    CompletionsResource,
    AsyncCompletionsResource,
    CompletionsResourceWithRawResponse,
    AsyncCompletionsResourceWithRawResponse,
    CompletionsResourceWithStreamingResponse,
    AsyncCompletionsResourceWithStreamingResponse,
)
from .deployments import (
    DeploymentsResource,
    AsyncDeploymentsResource,
    DeploymentsResourceWithRawResponse,
    AsyncDeploymentsResourceWithRawResponse,
    DeploymentsResourceWithStreamingResponse,
    AsyncDeploymentsResourceWithStreamingResponse,
)
from .chat_completions import (
    ChatCompletionsResource,
    AsyncChatCompletionsResource,
    ChatCompletionsResourceWithRawResponse,
    AsyncChatCompletionsResourceWithRawResponse,
    ChatCompletionsResourceWithStreamingResponse,
    AsyncChatCompletionsResourceWithStreamingResponse,
)
from .usage_statistics import (
    UsageStatisticsResource,
    AsyncUsageStatisticsResource,
    UsageStatisticsResourceWithRawResponse,
    AsyncUsageStatisticsResourceWithRawResponse,
    UsageStatisticsResourceWithStreamingResponse,
    AsyncUsageStatisticsResourceWithStreamingResponse,
)

__all__ = [
    "DeploymentsResource",
    "AsyncDeploymentsResource",
    "DeploymentsResourceWithRawResponse",
    "AsyncDeploymentsResourceWithRawResponse",
    "DeploymentsResourceWithStreamingResponse",
    "AsyncDeploymentsResourceWithStreamingResponse",
    "EmbeddingsResource",
    "AsyncEmbeddingsResource",
    "EmbeddingsResourceWithRawResponse",
    "AsyncEmbeddingsResourceWithRawResponse",
    "EmbeddingsResourceWithStreamingResponse",
    "AsyncEmbeddingsResourceWithStreamingResponse",
    "RerankingsResource",
    "AsyncRerankingsResource",
    "RerankingsResourceWithRawResponse",
    "AsyncRerankingsResourceWithRawResponse",
    "RerankingsResourceWithStreamingResponse",
    "AsyncRerankingsResourceWithStreamingResponse",
    "CompletionsResource",
    "AsyncCompletionsResource",
    "CompletionsResourceWithRawResponse",
    "AsyncCompletionsResourceWithRawResponse",
    "CompletionsResourceWithStreamingResponse",
    "AsyncCompletionsResourceWithStreamingResponse",
    "ChatCompletionsResource",
    "AsyncChatCompletionsResource",
    "ChatCompletionsResourceWithRawResponse",
    "AsyncChatCompletionsResourceWithRawResponse",
    "ChatCompletionsResourceWithStreamingResponse",
    "AsyncChatCompletionsResourceWithStreamingResponse",
    "UsageStatisticsResource",
    "AsyncUsageStatisticsResource",
    "UsageStatisticsResourceWithRawResponse",
    "AsyncUsageStatisticsResourceWithRawResponse",
    "UsageStatisticsResourceWithStreamingResponse",
    "AsyncUsageStatisticsResourceWithStreamingResponse",
    "ModelsResource",
    "AsyncModelsResource",
    "ModelsResourceWithRawResponse",
    "AsyncModelsResourceWithRawResponse",
    "ModelsResourceWithStreamingResponse",
    "AsyncModelsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SharePointDataSourceAuthConfigParam"]


class SharePointDataSourceAuthConfigParam(TypedDict, total=False):
    client_secret: Required[str]
    """Secret for the app registration associated with this SharePoint site"""

    source: Required[Literal["SharePoint"]]

    encrypted: bool



from __future__ import annotations

import os

import pytest

from scale_gp import SGPClient
from scale_gp.lib.dataset_builder import DatasetBuilder
from scale_gp.lib.types.multiturn import MultiturnTestCaseSchema
from scale_gp.lib.types.summarization import SummarizationTestCaseSchema
from scale_gp.types.evaluation_datasets import FlexibleTestCaseSchema

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDatasetBuilderLibrary:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_initialize_flexible(self, client: SGPClient) -> None:
        DatasetBuilder(client).initialize(
            account_id="account_id",
            name="dataset-builder",
            test_cases=[
                FlexibleTestCaseSchema(
                    input={"test-string": "string", "array": [1, 2, 3]},
                    expected_output={
                        "test-num": 1,
                        "object": {"foo": 1, "awesome": True},
                    },
                ),
                FlexibleTestCaseSchema(
                    input={"test-string": "string", "array": [1, 2, 3]},
                    expected_output={
                        "test-num": 1,
                        "object": {"foo": 1, "awesome": True},
                    },
                ),
            ],
        )

    @parametrize
    def test_initialize_summarization(self, client: SGPClient) -> None:
        DatasetBuilder(client).initialize(
            account_id="account_id",
            name="dataset-builder",
            test_cases=[
                SummarizationTestCaseSchema(document="test-document", expected_summary="expected-summary"),
                SummarizationTestCaseSchema(document="test-document", expected_summary="expected-summary"),
            ],
        )

    @parametrize
    def test_initialize_multiturn(self, client: SGPClient) -> None:
        DatasetBuilder(client).initialize(
            account_id="account_id",
            name="dataset-builder",
            test_cases=[
                MultiturnTestCaseSchema(
                    messages=[
                        {"role": "user", "content": "What is the capital of France?"},
                        {"role": "assistant", "content": "The capital of France is Paris."},
                        {"role": "user", "content": "Thank you!"},
                    ],
                    expected_messages=[
                        {"role": "user", "content": "What is the capital of France?"},
                        {"role": "assistant", "content": "The capital of France is Paris."},
                        {"role": "user", "content": "Thank you!"},
                    ],
                ),
                MultiturnTestCaseSchema(
                    messages=[
                        {"role": "user", "content": "What is the capital of France?"},
                        {"role": "assistant", "content": "The capital of France is Paris."},
                        {"role": "user", "content": "Thank you!"},
                    ],
                    turns=[1],
                    expected_messages=[
                        {"role": "user", "content": "What is the capital of France?"},
                        {"role": "assistant", "content": "The capital of France is Paris."},
                        {"role": "user", "content": "Thank you!"},
                    ],
                ),
            ],
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

__all__ = ["ApplicationTestCaseOutputRetrieveParams"]


class ApplicationTestCaseOutputRetrieveParams(TypedDict, total=False):
    view: List[Literal["MetricScores", "TestCaseVersion", "Trace"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.dashboard import Dashboard

__all__ = ["DashboardsResource", "AsyncDashboardsResource"]


class DashboardsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DashboardsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return DashboardsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DashboardsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return DashboardsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        dashboard_id: Literal[
            "top_level_dashboard", "engagements_dashboard", "executions_dashboard", "variant_overview_dashboard"
        ],
        *,
        application_spec_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Dashboard:
        """
        Get Dashboard Schema

        Args:
          dashboard_id: An enumeration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_spec_id:
            raise ValueError(
                f"Expected a non-empty value for `application_spec_id` but received {application_spec_id!r}"
            )
        if not dashboard_id:
            raise ValueError(f"Expected a non-empty value for `dashboard_id` but received {dashboard_id!r}")
        return self._get(
            f"/v4/applications/{application_spec_id}/dashboards/{dashboard_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Dashboard,
        )


class AsyncDashboardsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDashboardsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDashboardsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDashboardsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncDashboardsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        dashboard_id: Literal[
            "top_level_dashboard", "engagements_dashboard", "executions_dashboard", "variant_overview_dashboard"
        ],
        *,
        application_spec_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Dashboard:
        """
        Get Dashboard Schema

        Args:
          dashboard_id: An enumeration.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not application_spec_id:
            raise ValueError(
                f"Expected a non-empty value for `application_spec_id` but received {application_spec_id!r}"
            )
        if not dashboard_id:
            raise ValueError(f"Expected a non-empty value for `dashboard_id` but received {dashboard_id!r}")
        return await self._get(
            f"/v4/applications/{application_spec_id}/dashboards/{dashboard_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Dashboard,
        )


class DashboardsResourceWithRawResponse:
    def __init__(self, dashboards: DashboardsResource) -> None:
        self._dashboards = dashboards

        self.retrieve = to_raw_response_wrapper(
            dashboards.retrieve,
        )


class AsyncDashboardsResourceWithRawResponse:
    def __init__(self, dashboards: AsyncDashboardsResource) -> None:
        self._dashboards = dashboards

        self.retrieve = async_to_raw_response_wrapper(
            dashboards.retrieve,
        )


class DashboardsResourceWithStreamingResponse:
    def __init__(self, dashboards: DashboardsResource) -> None:
        self._dashboards = dashboards

        self.retrieve = to_streamed_response_wrapper(
            dashboards.retrieve,
        )


class AsyncDashboardsResourceWithStreamingResponse:
    def __init__(self, dashboards: AsyncDashboardsResource) -> None:
        self._dashboards = dashboards

        self.retrieve = async_to_streamed_response_wrapper(
            dashboards.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["FlexibleChunkParam"]


class FlexibleChunkParam(TypedDict, total=False):
    text: Required[str]

    metadata: object



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .test_case_result import TestCaseResult as TestCaseResult
from .task_update_params import TaskUpdateParams as TaskUpdateParams
from .contributor_metrics import ContributorMetrics as ContributorMetrics
from .test_case_result_with_views import TestCaseResultWithViews as TestCaseResultWithViews
from .test_case_result_list_params import TestCaseResultListParams as TestCaseResultListParams
from .paginated_contributor_metrics import PaginatedContributorMetrics as PaginatedContributorMetrics
from .test_case_result_batch_params import TestCaseResultBatchParams as TestCaseResultBatchParams
from .contributor_metric_list_params import ContributorMetricListParams as ContributorMetricListParams
from .test_case_result_create_params import TestCaseResultCreateParams as TestCaseResultCreateParams
from .test_case_result_update_params import TestCaseResultUpdateParams as TestCaseResultUpdateParams
from .test_case_result_batch_response import TestCaseResultBatchResponse as TestCaseResultBatchResponse
from .test_case_result_retrieve_params import TestCaseResultRetrieveParams as TestCaseResultRetrieveParams
from .paginated_test_case_result_with_views import PaginatedTestCaseResultWithViews as PaginatedTestCaseResultWithViews



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import application_schema_retrieve_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.application_schema_retrieve_response import ApplicationSchemaRetrieveResponse

__all__ = ["ApplicationSchemasResource", "AsyncApplicationSchemasResource"]


class ApplicationSchemasResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ApplicationSchemasResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return ApplicationSchemasResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ApplicationSchemasResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return ApplicationSchemasResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        version: Literal["OFFLINE", "V0", "AGENTS_SERVICE"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationSchemaRetrieveResponse:
        """
        Get Application Schemas

        Args:
          version: An enum representing the version states of an application and its nodes'
              schemas. Attributes: V0: The initial version of an application schema.
              AGENTS_SERVICE: Application schema references egp_services definition.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v4/application-schemas",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"version": version}, application_schema_retrieve_params.ApplicationSchemaRetrieveParams
                ),
            ),
            cast_to=ApplicationSchemaRetrieveResponse,
        )


class AsyncApplicationSchemasResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncApplicationSchemasResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncApplicationSchemasResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncApplicationSchemasResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncApplicationSchemasResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        version: Literal["OFFLINE", "V0", "AGENTS_SERVICE"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ApplicationSchemaRetrieveResponse:
        """
        Get Application Schemas

        Args:
          version: An enum representing the version states of an application and its nodes'
              schemas. Attributes: V0: The initial version of an application schema.
              AGENTS_SERVICE: Application schema references egp_services definition.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v4/application-schemas",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"version": version}, application_schema_retrieve_params.ApplicationSchemaRetrieveParams
                ),
            ),
            cast_to=ApplicationSchemaRetrieveResponse,
        )


class ApplicationSchemasResourceWithRawResponse:
    def __init__(self, application_schemas: ApplicationSchemasResource) -> None:
        self._application_schemas = application_schemas

        self.retrieve = to_raw_response_wrapper(
            application_schemas.retrieve,
        )


class AsyncApplicationSchemasResourceWithRawResponse:
    def __init__(self, application_schemas: AsyncApplicationSchemasResource) -> None:
        self._application_schemas = application_schemas

        self.retrieve = async_to_raw_response_wrapper(
            application_schemas.retrieve,
        )


class ApplicationSchemasResourceWithStreamingResponse:
    def __init__(self, application_schemas: ApplicationSchemasResource) -> None:
        self._application_schemas = application_schemas

        self.retrieve = to_streamed_response_wrapper(
            application_schemas.retrieve,
        )


class AsyncApplicationSchemasResourceWithStreamingResponse:
    def __init__(self, application_schemas: AsyncApplicationSchemasResource) -> None:
        self._application_schemas = application_schemas

        self.retrieve = async_to_streamed_response_wrapper(
            application_schemas.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.knowledge_bases import data_source_connection_delete_params
from ...types.knowledge_bases.delete_knowledge_base_data_source_connection_response import (
    DeleteKnowledgeBaseDataSourceConnectionResponse,
)

__all__ = ["DataSourceConnectionsResource", "AsyncDataSourceConnectionsResource"]


class DataSourceConnectionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DataSourceConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return DataSourceConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DataSourceConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return DataSourceConnectionsResourceWithStreamingResponse(self)

    def delete(
        self,
        knowledge_base_data_source_id: str,
        *,
        knowledge_base_id: str,
        dry_run: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DeleteKnowledgeBaseDataSourceConnectionResponse:
        """
        Delete Knowledge Base Data Source Connection

        Args:
          dry_run: Dryrun query parameter to determine first how many things will be deleted

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        if not knowledge_base_data_source_id:
            raise ValueError(
                f"Expected a non-empty value for `knowledge_base_data_source_id` but received {knowledge_base_data_source_id!r}"
            )
        return self._post(
            f"/v4/knowledge-bases/{knowledge_base_id}/data-source-connections/{knowledge_base_data_source_id}/delete",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"dry_run": dry_run}, data_source_connection_delete_params.DataSourceConnectionDeleteParams
                ),
            ),
            cast_to=DeleteKnowledgeBaseDataSourceConnectionResponse,
        )


class AsyncDataSourceConnectionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDataSourceConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDataSourceConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDataSourceConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncDataSourceConnectionsResourceWithStreamingResponse(self)

    async def delete(
        self,
        knowledge_base_data_source_id: str,
        *,
        knowledge_base_id: str,
        dry_run: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DeleteKnowledgeBaseDataSourceConnectionResponse:
        """
        Delete Knowledge Base Data Source Connection

        Args:
          dry_run: Dryrun query parameter to determine first how many things will be deleted

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_base_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_base_id` but received {knowledge_base_id!r}")
        if not knowledge_base_data_source_id:
            raise ValueError(
                f"Expected a non-empty value for `knowledge_base_data_source_id` but received {knowledge_base_data_source_id!r}"
            )
        return await self._post(
            f"/v4/knowledge-bases/{knowledge_base_id}/data-source-connections/{knowledge_base_data_source_id}/delete",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"dry_run": dry_run}, data_source_connection_delete_params.DataSourceConnectionDeleteParams
                ),
            ),
            cast_to=DeleteKnowledgeBaseDataSourceConnectionResponse,
        )


class DataSourceConnectionsResourceWithRawResponse:
    def __init__(self, data_source_connections: DataSourceConnectionsResource) -> None:
        self._data_source_connections = data_source_connections

        self.delete = to_raw_response_wrapper(
            data_source_connections.delete,
        )


class AsyncDataSourceConnectionsResourceWithRawResponse:
    def __init__(self, data_source_connections: AsyncDataSourceConnectionsResource) -> None:
        self._data_source_connections = data_source_connections

        self.delete = async_to_raw_response_wrapper(
            data_source_connections.delete,
        )


class DataSourceConnectionsResourceWithStreamingResponse:
    def __init__(self, data_source_connections: DataSourceConnectionsResource) -> None:
        self._data_source_connections = data_source_connections

        self.delete = to_streamed_response_wrapper(
            data_source_connections.delete,
        )


class AsyncDataSourceConnectionsResourceWithStreamingResponse:
    def __init__(self, data_source_connections: AsyncDataSourceConnectionsResource) -> None:
        self._data_source_connections = data_source_connections

        self.delete = async_to_streamed_response_wrapper(
            data_source_connections.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .evaluation_dataset_generation_job import EvaluationDatasetGenerationJob

__all__ = ["EvaluationDatasetGenerationJobsList"]


class EvaluationDatasetGenerationJobsList(BaseModel):
    generation_jobs: List[EvaluationDatasetGenerationJob]
    """List of evaluation dataset generation jobs."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["SlackDataSourceAuthConfigParam"]


class SlackDataSourceAuthConfigParam(TypedDict, total=False):
    bot_token: Required[str]
    """Your Slack app's Bot OAuth token. See https://api.slack.com/quickstart"""

    source: Required[Literal["Slack"]]

    encrypted: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ApplicationEdgeParam"]


class ApplicationEdgeParam(TypedDict, total=False):
    from_field: Required[str]

    from_node: Required[str]

    to_field: Required[str]

    to_node: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["ApplicationSpec"]


class ApplicationSpec(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    description: str
    """The description of the Application Spec"""

    name: str
    """The name of the Application Spec"""

    archived_at: Optional[datetime] = None
    """The date and time when the entity was archived in ISO format."""

    run_online_evaluation: Optional[bool] = None
    """Whether the application spec should run online evaluation, default is `false`"""

    theme_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .model_usage import ModelUsage as ModelUsage
from .completion_response import CompletionResponse as CompletionResponse
from .chunk_extra_info_schema import ChunkExtraInfoSchema as ChunkExtraInfoSchema
from .generic_delete_response import GenericDeleteResponse as GenericDeleteResponse
from .result_schema_generation import ResultSchemaGeneration as ResultSchemaGeneration
from .string_extra_info_schema import StringExtraInfoSchema as StringExtraInfoSchema



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = [
    "FineTuningJobCreateParams",
    "VendorConfiguration",
    "VendorConfigurationLaunchFineTuningJobConfiguration",
    "VendorConfigurationLlmEngineFineTuningJobConfiguration",
    "VendorConfigurationOpenAIFineTuningJobConfiguration",
]


class FineTuningJobCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    training_dataset_id: Required[str]

    base_model_id: str

    fine_tuned_model_id: str

    validation_dataset_id: str

    vendor_configuration: VendorConfiguration


class VendorConfigurationLaunchFineTuningJobConfiguration(TypedDict, total=False):
    hyperparameters: object

    output: str

    suffix: str

    vendor: Literal["LAUNCH"]

    wandb_config: object


class VendorConfigurationLlmEngineFineTuningJobConfiguration(TypedDict, total=False):
    hyperparameters: object

    output: str

    suffix: str

    vendor: Literal["LLMENGINE"]

    wandb_config: object


class VendorConfigurationOpenAIFineTuningJobConfiguration(TypedDict, total=False):
    hyperparameters: object

    suffix: str

    vendor: Literal["OPENAI"]


VendorConfiguration: TypeAlias = Union[
    VendorConfigurationLaunchFineTuningJobConfiguration,
    VendorConfigurationLlmEngineFineTuningJobConfiguration,
    VendorConfigurationOpenAIFineTuningJobConfiguration,
]



### application_report_card_example_script.py ###
# This script demonstrates how to create an application report card for an application variant
# A report card will provide a summary of the performance of the application variant against an evaluation dataset
# Three main scores will be produced: Accuracy, Quality and Trust & Safety.
# The performance in each category will contribute to an overall Scale Confidence Score for the variant.


# You can view generated report cards in the UI at:
# https://egp.dashboard.scale.com/applications/<application_spec_id>/<application_variant_id>/report-card/overview

import os
import time

from scale_gp import SGPClient
from scale_gp.types.application_edge_param import ApplicationEdgeParam
from scale_gp.types.application_node_param import ApplicationNodeParam
from scale_gp.types.model_template_create_params import (
    VendorConfiguration,
    VendorConfigurationBundleConfig,
    VendorConfigurationEndpointConfig,
)
from scale_gp.types.application_configuration_param import ApplicationConfigurationParam

account_id = os.environ.get("SGP_ACCOUNT_ID", None)
api_key = os.environ.get("SGP_API_KEY", None)

assert (
    account_id is not None
), "You need to set the SGP_ACCOUNT_ID - you can find it at https://egp.dashboard.scale.com/admin/accounts"
assert api_key is not None, "You need to provide your API key - see https://egp.dashboard.scale.com/admin/api-key"

# Create an SGP Client
client = SGPClient(api_key=api_key)

### SETUP ###
bundle_config = VendorConfigurationBundleConfig(image="gemini-pro", registry="aws-registry", tag="latest")
endpoint_config = VendorConfigurationEndpointConfig(
    max_workers=3,
)

vendor_configuration = VendorConfiguration(
    bundle_config=bundle_config,
    endpoint_config=endpoint_config,
)
model_template = client.model_templates.create(
    account_id=account_id,
    endpoint_type="SYNC",
    model_type="COMPLETION",
    name="Gemini-Pro Template",
    vendor_configuration=vendor_configuration,
)

model_instance = client.models.create(
    account_id=account_id,
    model_type="COMPLETION",
    name="gemini-pro",
    model_vendor="GOOGLE",
    model_template_id=model_template.id,
)
model_deployment = client.models.deployments.create(
    model_instance_id=model_instance.id, name="Gemini-Pro Deployment", account_id=account_id
)

input_node_id = "input_node"
completion_node_id = "completion_node"
output_node_id = "output_node"
application_spec = client.application_specs.create(
    account_id=account_id,
    description="Test application spec",
    name="test-application-spec" + str(time.time()),
)
edges = [
    ApplicationEdgeParam(
        from_node=input_node_id,
        to_node=completion_node_id,
        from_field="input-text",
        to_field="prompt",
    ),
    ApplicationEdgeParam(
        from_node=completion_node_id,
        to_node=output_node_id,
        from_field="completion",
        to_field="output-text",
    ),
]

nodes = [
    ApplicationNodeParam(
        id=input_node_id,
        application_node_schema_id="text_input_schema",
        configuration={"input-text": {"type": "TEXT", "value": "Completion prompt"}},
    ),
    ApplicationNodeParam(
        id=completion_node_id,
        application_node_schema_id="completion_model_schema",
        configuration={
            "completion_model_id": {
                "type": "TEXT",
                "value": model_deployment.id,  # define the deployed completion model we want to use inside the application
            }
        },
    ),
    ApplicationNodeParam(
        id=output_node_id,
        application_node_schema_id="text_output_schema",
        configuration={"completion": {"type": "TEXT", "value": "completion result"}},
    ),
]
configuration = ApplicationConfigurationParam(edges=edges, nodes=nodes)
application_variant = client.application_variants.create(
    account_id=account_id,
    application_spec_id=application_spec.id,
    name="test-application-variant",
    version="V0",
    description="Test application variant",
    configuration=configuration,
)

DATASET = [
    {
        "input": "What is the capital of France?",
        "expected_output": "The capital of France is Paris.",
        "expected_extra_info": {
            "info": "Paris, located in the northern part of France along the Seine River, is not only the country’s capital but also its largest city. Known for its art, fashion, gastronomy, and culture, Paris has a significant influence both in France and globally.",
            "schema_type": "STRING",
        },
    },
    {
        "input": "What color is an apple?",
        "expected_output": "An apple is typically red when ripe, though green and yellow varieties also exist depending on the species and maturity.",
        "expected_extra_info": {
            "info": "Apples come in various colors including red, green, and yellow. Red apples are one of the most common, with varieties like Red Delicious being very popular. Green apples, like Granny Smith, are tart and used often in baking. Yellow apples such as Golden Delicious are sweet and softer.",
            "schema_type": "STRING",
        },
    },
    {
        "input": "Who is the first president of the USA?",
        "expected_output": "The first president of the USA is George Washington.",
        "expected_extra_info": {
            "info": "George Washington served as the first president of the United States from 1789 to 1797. He is a pivotal figure in American history, recognized for his leadership during the Revolutionary War and setting many precedents for the national government.",
            "schema_type": "STRING",
        },
    },
]

evaluation_dataset = client.evaluation_datasets.create(
    account_id=account_id,
    name="evaluation_dataset",
    schema_type="GENERATION",
    type="manual",
)
uploaded_test_cases = client.evaluation_datasets.test_cases.batch(
    evaluation_dataset_id=evaluation_dataset.id,
    items=[{"account_id": account_id, "schema_type": "GENERATION", "test_case_data": item} for item in DATASET],
)

manual_dataset_version = client.evaluation_datasets.evaluation_dataset_versions.create(
    evaluation_dataset_id=evaluation_dataset.id
)

published_dataset_response = client.evaluation_datasets.publish(
    evaluation_dataset_id=evaluation_dataset.id,
)
### SETUP END ###

# create an application variant report
application_variant_report_create_response = client.application_variant_reports.create(
    application_variant_id=application_variant.id,
    evaluation_dataset_ids=[evaluation_dataset.id],
    account_id=account_id,
)

# retrieve the application variant report, will still be PENDING if immediately retrieved
while True:
    application_variant_report = client.application_variant_reports.retrieve(
        application_variant_report_id=application_variant_report_create_response.id,
        view="AsyncJobs",
    )
    if application_variant_report.async_jobs[0].status != "Completed":
        print("Creating application variant report... this might take a while")
        time.sleep(20)
    else:
        break

print("Generated application variant report:")
print(application_variant_report)



import logging
from enum import Enum
from typing_extensions import Literal

from scale_gp.types.application_edge_param import ApplicationEdgeParam
from scale_gp.types.application_node_param import ApplicationNodeParam
from scale_gp.types.application_configuration_param import ApplicationConfigurationParam

from .._client import SGPClient
from .._models import BaseModel

log: logging.Logger = logging.getLogger("scale_gp")


class ApplicationTemplate(BaseModel):
    """
    ### Description
    ApplicationTemplate describes the pre-defined available application templates that can be created by the ApplicationBuilder
    ### Parameters
    - application_type: The type of the predefined application template, currently supports "COMPLETION" and "RAG"
    - variant_id: The id of the application variant created by the ApplicationBuilder
    """

    application_type: Literal["COMPLETION", "RAG"]
    variant_id: str


class ApplicationBuilderNodeIDs(Enum):
    input_node_id = "input_node"
    completion_node_id = "completion_node"
    output_node_id = "output_node"
    prompt_engineering_node_id = "prompt_engineering_node"
    reranker_node_id = "reranker_node"
    knowledge_base_node_id = "knowledge_base_node"


class ApplicationBuilder:
    """
    ### Description
    ApplicationBuilder helps users quickly create predefined applications based on templates
    From the template, users can also easily call the process function without having to know what input fields are required
    """

    def __init__(self, client: SGPClient):
        self.client = client
        self.application = None

    def create_completion_application(
        self, *, account_id: str, application_name: str, completion_model_id: str
    ) -> ApplicationTemplate:
        """
        ### Description
        Creates a simple completion or chat completion application, the equivalent of Input -> Completion Model -> Output

        ### Parameters
        - account_id: The account id of the user
        - application_name: The name of the application
        - completion_model_id: The model deployment id of the completion model to use
        """

        # create application spec, this is the base for the application variant
        application_spec_id = self._create_app_spec(account_id=account_id, application_name=application_name)

        # create application variant config for a simple completion app
        edges = [
            ApplicationEdgeParam(
                from_node=ApplicationBuilderNodeIDs.input_node_id.value,
                to_node=ApplicationBuilderNodeIDs.completion_node_id.value,
                from_field="input-text",
                to_field="prompt",
            ),
            ApplicationEdgeParam(
                from_node=ApplicationBuilderNodeIDs.completion_node_id.value,
                to_node=ApplicationBuilderNodeIDs.output_node_id.value,
                from_field="completion",
                to_field="output-text",
            ),
        ]

        nodes = [
            ApplicationNodeParam(
                id=ApplicationBuilderNodeIDs.input_node_id.value,
                application_node_schema_id="text_input_schema",
                configuration={"input-text": {"value": "Completion prompt"}},
            ),
            ApplicationNodeParam(
                id=ApplicationBuilderNodeIDs.completion_node_id.value,
                application_node_schema_id="completion_model_schema",
                configuration={
                    "completion_model_id": {
                        "value": completion_model_id,
                    }
                },
            ),
            ApplicationNodeParam(
                id=ApplicationBuilderNodeIDs.output_node_id.value,
                application_node_schema_id="text_output_schema",
                configuration={"completion": {"type": "TEXT", "value": "completion result"}},
            ),
        ]
        configuration = ApplicationConfigurationParam(edges=edges, nodes=nodes)

        application_variant = self.client.application_variants.create(
            account_id=account_id,
            application_spec_id=application_spec_id,
            name=application_name,
            version="V0",
            description="Automatically created completion application variant for " + application_name,
            configuration=configuration,
        )
        self.application = ApplicationTemplate(application_type="COMPLETION", variant_id=application_variant.id)
        log.info(f"\nCreated completion application for account {account_id}\nApp Variant ID: {application_variant.id}")
        return self.application

    def create_rag_application(
        self,
        *,
        account_id: str,
        application_name: str,
        knowledge_base_id: str,
        knowledge_base_top_k: int,
        reranker_model_id: str,
        reranker_top_k: int,
        prompt_engineering_template: str,
        completion_model_id: str,
    ) -> ApplicationTemplate:
        """
        ### Description
        Creates a simple RAG application, the equivalent of Input -> Knowledge Base -> Reranker -> Prompt Engineering -> Completion Model -> Output

        ### Parameters
        - account_id: The account id of the user
        - application_name: The name of the application
        - knowledge_base_id: The knowledge base id to use
        - knowledge_base_top_k: The top k parameter to use when querying the knowledge base
        - reranker_model_id: The model deployment id of the reranker model to use
        - reranker_top_k: The top k parameter to use with the reranker model
        - prompt_engineering_template: The prompt engineering template to use. Refer to https://mustache.github.io/mustache.5.html for more details
        - completion_model_id: The model deployment id of the completion model to use
        """

        # create application spec which is the base for the application variant
        application_spec_id = self._create_app_spec(account_id=account_id, application_name=application_name)

        edges = [
            ApplicationEdgeParam(
                from_node=ApplicationBuilderNodeIDs.input_node_id.value,
                to_node=ApplicationBuilderNodeIDs.knowledge_base_node_id.value,
                from_field="input-text",
                to_field="query",
            ),
            ApplicationEdgeParam(
                from_node=ApplicationBuilderNodeIDs.input_node_id.value,
                to_node=ApplicationBuilderNodeIDs.prompt_engineering_node_id.value,
                from_field="input-text",
                to_field="query",
            ),
            ApplicationEdgeParam(
                from_node=ApplicationBuilderNodeIDs.knowledge_base_node_id.value,
                to_node=ApplicationBuilderNodeIDs.reranker_node_id.value,
                from_field="chunks",
                to_field="chunks",
            ),
            ApplicationEdgeParam(
                from_node=ApplicationBuilderNodeIDs.input_node_id.value,
                to_node=ApplicationBuilderNodeIDs.reranker_node_id.value,
                from_field="input-text",
                to_field="prompt",
            ),
            ApplicationEdgeParam(
                from_node=ApplicationBuilderNodeIDs.reranker_node_id.value,
                to_node=ApplicationBuilderNodeIDs.prompt_engineering_node_id.value,
                from_field="chunks",
                to_field="chunks",
            ),
            ApplicationEdgeParam(
                from_node=ApplicationBuilderNodeIDs.prompt_engineering_node_id.value,
                to_node=ApplicationBuilderNodeIDs.completion_node_id.value,
                from_field="prompt",
                to_field="prompt",
            ),
            ApplicationEdgeParam(
                from_node=ApplicationBuilderNodeIDs.completion_node_id.value,
                to_node=ApplicationBuilderNodeIDs.output_node_id.value,
                from_field="completion",
                to_field="output-text",
            ),
            ApplicationEdgeParam(
                from_node=ApplicationBuilderNodeIDs.reranker_node_id.value,
                to_node=ApplicationBuilderNodeIDs.output_node_id.value,
                from_field="chunks",
                to_field="metadata",
            ),
        ]

        nodes = [
            ApplicationNodeParam(
                id=ApplicationBuilderNodeIDs.prompt_engineering_node_id.value,
                application_node_schema_id="prompt_engineering_schema",
                configuration={"prompt_template": {"value": "TEXT", "value": prompt_engineering_template}},
            ),
            ApplicationNodeParam(
                id=ApplicationBuilderNodeIDs.input_node_id.value,
                application_node_schema_id="text_input_schema",
                configuration={"input-text": {"value": "input-text"}},
            ),
            ApplicationNodeParam(
                id=ApplicationBuilderNodeIDs.completion_node_id.value,
                application_node_schema_id="completion_model_schema",
                configuration={
                    "completion_model_id": {
                        "value": completion_model_id,
                    }
                },
            ),
            ApplicationNodeParam(
                id=ApplicationBuilderNodeIDs.knowledge_base_node_id.value,
                application_node_schema_id="knowledge_base_schema",
                configuration={
                    "kb_id": {
                        "value": knowledge_base_id,
                    },
                    "top_k": {
                        "value": knowledge_base_top_k,
                    },
                },
            ),
            ApplicationNodeParam(
                id=ApplicationBuilderNodeIDs.reranker_node_id.value,
                application_node_schema_id="reranker_schema",
                configuration={
                    "reranking_model_id": {
                        "value": reranker_model_id,
                    },
                    "top_k": {
                        "value": reranker_top_k,
                    },
                },
            ),
            ApplicationNodeParam(
                id=ApplicationBuilderNodeIDs.output_node_id.value,
                application_node_schema_id="text_output_schema",
                configuration={"completion": {"value": "completion result"}},
            ),
        ]
        configuration = ApplicationConfigurationParam(edges=edges, nodes=nodes)

        application_variant = self.client.application_variants.create(
            account_id=account_id,
            application_spec_id=application_spec_id,
            name=application_name,
            version="V0",
            description="Automatically created rag application variant for " + application_name,
            configuration=configuration,
        )
        self.application = ApplicationTemplate(application_type="RAG", variant_id=application_variant.id)
        log.info(f"\nCreated RAG application for account {account_id}\nApp Variant ID: {application_variant.id}")
        return self.application

    def process(self, *, query: str) -> str:
        if not self.application:
            raise ValueError(
                "Application not created - you need to create the app with one of the create_ methods on this class before processing."
            )

        # call the application based on the application type
        if self.application.application_type == "COMPLETION":
            variant_response = self.client.application_variants.process(
                application_variant_id=self.application.variant_id,
                inputs={ApplicationBuilderNodeIDs.input_node_id.value: {"input-text": query}},
            )
            completion_response = variant_response["output-text"]
            log.info(f"\nProcessed completion application\nApp Variant ID: {self.application.variant_id}")
            return completion_response
        elif self.application.application_type == "RAG":
            variant_response = self.client.application_variants.process(
                application_variant_id=self.application.variant_id,
                inputs={ApplicationBuilderNodeIDs.input_node_id.value: {"input-text": query}},
            )
            rag_response = variant_response["output-text"]
            log.info(f"\nProcessed RAG application\nApp Variant ID: {self.application.variant_id}")
            return rag_response
        else:
            raise ValueError("Unsupported application type")

    def _create_app_spec(self, *, account_id: str, application_name: str) -> str:
        application_spec = self.client.application_specs.create(
            account_id=account_id,
            description="Automatically created applicaton spec for " + application_name,
            name=application_name,
        )
        return application_spec.id



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["AnnotationConfigParam", "Component"]


class Component(TypedDict, total=False):
    data_loc: Required[List[str]]

    label: str


class AnnotationConfigParam(TypedDict, total=False):
    components: Required[Iterable[Iterable[Component]]]

    annotation_config_type: Literal["flexible", "summarization", "multiturn"]
    """An enumeration."""

    direction: Literal["col", "row"]
    """An enumeration."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, overload
from typing_extensions import Literal

import httpx

from .tasks import (
    TasksResource,
    AsyncTasksResource,
    TasksResourceWithRawResponse,
    AsyncTasksResourceWithRawResponse,
    TasksResourceWithStreamingResponse,
    AsyncTasksResourceWithStreamingResponse,
)
from ...types import (
    evaluation_list_params,
    evaluation_create_params,
    evaluation_update_params,
    evaluation_retrieve_params,
)
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    required_args,
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncPageResponse, AsyncPageResponse
from ...types.task import Task
from ..._base_client import AsyncPaginator, make_request_options
from .test_case_results import (
    TestCaseResultsResource,
    AsyncTestCaseResultsResource,
    TestCaseResultsResourceWithRawResponse,
    AsyncTestCaseResultsResourceWithRawResponse,
    TestCaseResultsResourceWithStreamingResponse,
    AsyncTestCaseResultsResourceWithStreamingResponse,
)
from ...types.evaluation import Evaluation
from .evaluation_metrics import (
    EvaluationMetricsResource,
    AsyncEvaluationMetricsResource,
    EvaluationMetricsResourceWithRawResponse,
    AsyncEvaluationMetricsResourceWithRawResponse,
    EvaluationMetricsResourceWithStreamingResponse,
    AsyncEvaluationMetricsResourceWithStreamingResponse,
)
from .contributor_metrics import (
    ContributorMetricsResource,
    AsyncContributorMetricsResource,
    ContributorMetricsResourceWithRawResponse,
    AsyncContributorMetricsResourceWithRawResponse,
    ContributorMetricsResourceWithStreamingResponse,
    AsyncContributorMetricsResourceWithStreamingResponse,
)
from .hybrid_eval_metrics import (
    HybridEvalMetricsResource,
    AsyncHybridEvalMetricsResource,
    HybridEvalMetricsResourceWithRawResponse,
    AsyncHybridEvalMetricsResourceWithRawResponse,
    HybridEvalMetricsResourceWithStreamingResponse,
    AsyncHybridEvalMetricsResourceWithStreamingResponse,
)
from ...types.evaluation_with_views import EvaluationWithViews
from ...types.annotation_config_param import AnnotationConfigParam
from .test_case_results.test_case_results import TestCaseResultsResource, AsyncTestCaseResultsResource
from ...types.shared.generic_delete_response import GenericDeleteResponse

__all__ = ["EvaluationsResource", "AsyncEvaluationsResource"]


class EvaluationsResource(SyncAPIResource):
    @cached_property
    def tasks(self) -> TasksResource:
        return TasksResource(self._client)

    @cached_property
    def contributor_metrics(self) -> ContributorMetricsResource:
        return ContributorMetricsResource(self._client)

    @cached_property
    def evaluation_metrics(self) -> EvaluationMetricsResource:
        return EvaluationMetricsResource(self._client)

    @cached_property
    def hybrid_eval_metrics(self) -> HybridEvalMetricsResource:
        return HybridEvalMetricsResource(self._client)

    @cached_property
    def test_case_results(self) -> TestCaseResultsResource:
        return TestCaseResultsResource(self._client)

    @cached_property
    def with_raw_response(self) -> EvaluationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return EvaluationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EvaluationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return EvaluationsResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        account_id: str,
        application_spec_id: str,
        application_variant_id: str,
        description: str,
        evaluation_dataset_id: str,
        name: str,
        annotation_config: evaluation_create_params.EvaluationBuilderRequestAnnotationConfig | NotGiven = NOT_GIVEN,
        evaluation_config: object | NotGiven = NOT_GIVEN,
        evaluation_config_id: str | NotGiven = NOT_GIVEN,
        evaluation_dataset_version: int | NotGiven = NOT_GIVEN,
        question_id_to_annotation_config: Dict[str, AnnotationConfigParam] | NotGiven = NOT_GIVEN,
        tags: object | NotGiven = NOT_GIVEN,
        type: Literal["builder"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Evaluation:
        """
        ### Description

        Creates a evaluation

        ### Details

        This API can be used to create a evaluation. To use this API, review the request
        schema and pass in all fields that are required to create a evaluation.

        Args:
          account_id: The ID of the account that owns the given entity.

          annotation_config: Annotation configuration for tasking

          evaluation_config_id: The ID of the associated evaluation config.

          question_id_to_annotation_config: Specifies the annotation configuration to use for specific questions.

          type: create standalone evaluation or build evaluation which will auto generate test
              case results

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        account_id: str,
        application_spec_id: str,
        description: str,
        name: str,
        annotation_config: evaluation_create_params.DefaultEvaluationRequestAnnotationConfig | NotGiven = NOT_GIVEN,
        application_variant_id: str | NotGiven = NOT_GIVEN,
        evaluation_config: object | NotGiven = NOT_GIVEN,
        evaluation_config_id: str | NotGiven = NOT_GIVEN,
        question_id_to_annotation_config: Dict[str, AnnotationConfigParam] | NotGiven = NOT_GIVEN,
        tags: object | NotGiven = NOT_GIVEN,
        type: Literal["default"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Evaluation:
        """
        ### Description

        Creates a evaluation

        ### Details

        This API can be used to create a evaluation. To use this API, review the request
        schema and pass in all fields that are required to create a evaluation.

        Args:
          account_id: The ID of the account that owns the given entity.

          annotation_config: Annotation configuration for tasking

          evaluation_config_id: The ID of the associated evaluation config.

          question_id_to_annotation_config: Specifies the annotation configuration to use for specific questions.

          type: create standalone evaluation or build evaluation which will auto generate test
              case results

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(
        ["account_id", "application_spec_id", "application_variant_id", "description", "evaluation_dataset_id", "name"],
        ["account_id", "application_spec_id", "description", "name"],
    )
    def create(
        self,
        *,
        account_id: str,
        application_spec_id: str,
        application_variant_id: str | NotGiven = NOT_GIVEN,
        description: str,
        evaluation_dataset_id: str | NotGiven = NOT_GIVEN,
        name: str,
        annotation_config: evaluation_create_params.EvaluationBuilderRequestAnnotationConfig | NotGiven = NOT_GIVEN,
        evaluation_config: object | NotGiven = NOT_GIVEN,
        evaluation_config_id: str | NotGiven = NOT_GIVEN,
        evaluation_dataset_version: int | NotGiven = NOT_GIVEN,
        question_id_to_annotation_config: Dict[str, AnnotationConfigParam] | NotGiven = NOT_GIVEN,
        tags: object | NotGiven = NOT_GIVEN,
        type: Literal["builder"] | Literal["default"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Evaluation:
        return self._post(
            "/v4/evaluations",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "application_spec_id": application_spec_id,
                    "application_variant_id": application_variant_id,
                    "description": description,
                    "evaluation_dataset_id": evaluation_dataset_id,
                    "name": name,
                    "annotation_config": annotation_config,
                    "evaluation_config": evaluation_config,
                    "evaluation_config_id": evaluation_config_id,
                    "evaluation_dataset_version": evaluation_dataset_version,
                    "question_id_to_annotation_config": question_id_to_annotation_config,
                    "tags": tags,
                    "type": type,
                },
                evaluation_create_params.EvaluationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Evaluation,
        )

    def retrieve(
        self,
        evaluation_id: str,
        *,
        view: List[Literal["ApplicationSpec", "AsyncJobs", "EvaluationConfig", "TestCaseResults"]]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationWithViews:
        """
        ### Description

        Gets the details of a evaluation

        ### Details

        This API can be used to get information about a single evaluation by ID. To use
        this API, pass in the `id` that was returned from your Create Evaluation API
        call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        return self._get(
            f"/v4/evaluations/{evaluation_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"view": view}, evaluation_retrieve_params.EvaluationRetrieveParams),
            ),
            cast_to=EvaluationWithViews,
        )

    @overload
    def update(
        self,
        evaluation_id: str,
        *,
        annotation_config: evaluation_update_params.PartialPatchEvaluationRequestAnnotationConfig
        | NotGiven = NOT_GIVEN,
        application_spec_id: str | NotGiven = NOT_GIVEN,
        application_variant_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        evaluation_config: object | NotGiven = NOT_GIVEN,
        evaluation_config_id: str | NotGiven = NOT_GIVEN,
        evaluation_type: Literal["llm_benchmark"] | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        question_id_to_annotation_config: Dict[str, AnnotationConfigParam] | NotGiven = NOT_GIVEN,
        restore: Literal[False] | NotGiven = NOT_GIVEN,
        tags: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Evaluation:
        """
        ### Description

        Updates a evaluation

        ### Details

        This API can be used to update the evaluation that matches the ID that was
        passed in as a path parameter. To use this API, pass in the `id` that was
        returned from your Create Evaluation API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          annotation_config: Annotation configuration for tasking

          evaluation_config_id: The ID of the associated evaluation config.

          evaluation_type: If llm_benchmark is provided, the evaluation will be updated to a hybrid
              evaluation. No-op on existing hybrid evaluations, and not available for studio
              evaluations.

          question_id_to_annotation_config: Specifies the annotation configuration to use for specific questions.

          restore: Set to true to restore the entity from the database.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def update(
        self,
        evaluation_id: str,
        *,
        restore: Literal[True],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Evaluation:
        """
        ### Description

        Updates a evaluation

        ### Details

        This API can be used to update the evaluation that matches the ID that was
        passed in as a path parameter. To use this API, pass in the `id` that was
        returned from your Create Evaluation API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          restore: Set to true to restore the entity from the database.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    def update(
        self,
        evaluation_id: str,
        *,
        annotation_config: evaluation_update_params.PartialPatchEvaluationRequestAnnotationConfig
        | NotGiven = NOT_GIVEN,
        application_spec_id: str | NotGiven = NOT_GIVEN,
        application_variant_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        evaluation_config: object | NotGiven = NOT_GIVEN,
        evaluation_config_id: str | NotGiven = NOT_GIVEN,
        evaluation_type: Literal["llm_benchmark"] | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        question_id_to_annotation_config: Dict[str, AnnotationConfigParam] | NotGiven = NOT_GIVEN,
        restore: Literal[False] | Literal[True] | NotGiven = NOT_GIVEN,
        tags: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Evaluation:
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        return self._patch(
            f"/v4/evaluations/{evaluation_id}",
            body=maybe_transform(
                {
                    "annotation_config": annotation_config,
                    "application_spec_id": application_spec_id,
                    "application_variant_id": application_variant_id,
                    "description": description,
                    "evaluation_config": evaluation_config,
                    "evaluation_config_id": evaluation_config_id,
                    "evaluation_type": evaluation_type,
                    "name": name,
                    "question_id_to_annotation_config": question_id_to_annotation_config,
                    "restore": restore,
                    "tags": tags,
                },
                evaluation_update_params.EvaluationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Evaluation,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        application_spec_id: Union[int, str] | NotGiven = NOT_GIVEN,
        include_archived: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        sort_by: List[
            Literal[
                "status:asc",
                "status:desc",
                "application_spec_id:asc",
                "application_spec_id:desc",
                "application_spec:asc",
                "application_spec:desc",
                "application_variant_id:asc",
                "application_variant_id:desc",
                "application_variant:asc",
                "application_variant:desc",
                "evaluation_config_id:asc",
                "evaluation_config_id:desc",
                "completed_at:asc",
                "completed_at:desc",
                "total_test_case_result_count:asc",
                "total_test_case_result_count:desc",
                "completed_test_case_result_count:asc",
                "completed_test_case_result_count:desc",
                "annotation_config:asc",
                "annotation_config:desc",
                "question_id_to_annotation_config:asc",
                "question_id_to_annotation_config:desc",
                "evaluation_config_expanded:asc",
                "evaluation_config_expanded:desc",
                "test_case_results:asc",
                "test_case_results:desc",
                "async_jobs:asc",
                "async_jobs:desc",
                "id:asc",
                "id:desc",
                "created_at:asc",
                "created_at:desc",
                "account_id:asc",
                "account_id:desc",
                "created_by_user_id:asc",
                "created_by_user_id:desc",
                "archived_at:asc",
                "archived_at:desc",
                "created_by_user:asc",
                "created_by_user:desc",
                "name:asc",
                "name:desc",
                "description:asc",
                "description:desc",
                "tags:asc",
                "tags:desc",
                "evaluation_config:asc",
                "evaluation_config:desc",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        view: List[Literal["ApplicationSpec", "AsyncJobs", "EvaluationConfig", "TestCaseResults"]]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncPageResponse[EvaluationWithViews]:
        """
        ### Description

        Lists all evaluations accessible to the user.

        ### Details

        This API can be used to list evaluations. If a user has access to multiple
        accounts, all evaluations from all accounts the user is associated with will be
        returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/evaluations",
            page=SyncPageResponse[EvaluationWithViews],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "application_spec_id": application_spec_id,
                        "include_archived": include_archived,
                        "limit": limit,
                        "page": page,
                        "sort_by": sort_by,
                        "view": view,
                    },
                    evaluation_list_params.EvaluationListParams,
                ),
            ),
            model=EvaluationWithViews,
        )

    def delete(
        self,
        evaluation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a evaluation

        ### Details

        This API can be used to delete a evaluation by ID. To use this API, pass in the
        `id` that was returned from your Create Evaluation API call as a path parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        return self._delete(
            f"/v4/evaluations/{evaluation_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )

    def claim_task(
        self,
        evaluation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Task:
        """
        Claim Evaluation Task

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        return self._post(
            f"/v4/evaluations/{evaluation_id}/claim-task",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Task,
        )


class AsyncEvaluationsResource(AsyncAPIResource):
    @cached_property
    def tasks(self) -> AsyncTasksResource:
        return AsyncTasksResource(self._client)

    @cached_property
    def contributor_metrics(self) -> AsyncContributorMetricsResource:
        return AsyncContributorMetricsResource(self._client)

    @cached_property
    def evaluation_metrics(self) -> AsyncEvaluationMetricsResource:
        return AsyncEvaluationMetricsResource(self._client)

    @cached_property
    def hybrid_eval_metrics(self) -> AsyncHybridEvalMetricsResource:
        return AsyncHybridEvalMetricsResource(self._client)

    @cached_property
    def test_case_results(self) -> AsyncTestCaseResultsResource:
        return AsyncTestCaseResultsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncEvaluationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEvaluationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEvaluationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncEvaluationsResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        account_id: str,
        application_spec_id: str,
        application_variant_id: str,
        description: str,
        evaluation_dataset_id: str,
        name: str,
        annotation_config: evaluation_create_params.EvaluationBuilderRequestAnnotationConfig | NotGiven = NOT_GIVEN,
        evaluation_config: object | NotGiven = NOT_GIVEN,
        evaluation_config_id: str | NotGiven = NOT_GIVEN,
        evaluation_dataset_version: int | NotGiven = NOT_GIVEN,
        question_id_to_annotation_config: Dict[str, AnnotationConfigParam] | NotGiven = NOT_GIVEN,
        tags: object | NotGiven = NOT_GIVEN,
        type: Literal["builder"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Evaluation:
        """
        ### Description

        Creates a evaluation

        ### Details

        This API can be used to create a evaluation. To use this API, review the request
        schema and pass in all fields that are required to create a evaluation.

        Args:
          account_id: The ID of the account that owns the given entity.

          annotation_config: Annotation configuration for tasking

          evaluation_config_id: The ID of the associated evaluation config.

          question_id_to_annotation_config: Specifies the annotation configuration to use for specific questions.

          type: create standalone evaluation or build evaluation which will auto generate test
              case results

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        account_id: str,
        application_spec_id: str,
        description: str,
        name: str,
        annotation_config: evaluation_create_params.DefaultEvaluationRequestAnnotationConfig | NotGiven = NOT_GIVEN,
        application_variant_id: str | NotGiven = NOT_GIVEN,
        evaluation_config: object | NotGiven = NOT_GIVEN,
        evaluation_config_id: str | NotGiven = NOT_GIVEN,
        question_id_to_annotation_config: Dict[str, AnnotationConfigParam] | NotGiven = NOT_GIVEN,
        tags: object | NotGiven = NOT_GIVEN,
        type: Literal["default"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Evaluation:
        """
        ### Description

        Creates a evaluation

        ### Details

        This API can be used to create a evaluation. To use this API, review the request
        schema and pass in all fields that are required to create a evaluation.

        Args:
          account_id: The ID of the account that owns the given entity.

          annotation_config: Annotation configuration for tasking

          evaluation_config_id: The ID of the associated evaluation config.

          question_id_to_annotation_config: Specifies the annotation configuration to use for specific questions.

          type: create standalone evaluation or build evaluation which will auto generate test
              case results

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(
        ["account_id", "application_spec_id", "application_variant_id", "description", "evaluation_dataset_id", "name"],
        ["account_id", "application_spec_id", "description", "name"],
    )
    async def create(
        self,
        *,
        account_id: str,
        application_spec_id: str,
        application_variant_id: str | NotGiven = NOT_GIVEN,
        description: str,
        evaluation_dataset_id: str | NotGiven = NOT_GIVEN,
        name: str,
        annotation_config: evaluation_create_params.EvaluationBuilderRequestAnnotationConfig | NotGiven = NOT_GIVEN,
        evaluation_config: object | NotGiven = NOT_GIVEN,
        evaluation_config_id: str | NotGiven = NOT_GIVEN,
        evaluation_dataset_version: int | NotGiven = NOT_GIVEN,
        question_id_to_annotation_config: Dict[str, AnnotationConfigParam] | NotGiven = NOT_GIVEN,
        tags: object | NotGiven = NOT_GIVEN,
        type: Literal["builder"] | Literal["default"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Evaluation:
        return await self._post(
            "/v4/evaluations",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "application_spec_id": application_spec_id,
                    "application_variant_id": application_variant_id,
                    "description": description,
                    "evaluation_dataset_id": evaluation_dataset_id,
                    "name": name,
                    "annotation_config": annotation_config,
                    "evaluation_config": evaluation_config,
                    "evaluation_config_id": evaluation_config_id,
                    "evaluation_dataset_version": evaluation_dataset_version,
                    "question_id_to_annotation_config": question_id_to_annotation_config,
                    "tags": tags,
                    "type": type,
                },
                evaluation_create_params.EvaluationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Evaluation,
        )

    async def retrieve(
        self,
        evaluation_id: str,
        *,
        view: List[Literal["ApplicationSpec", "AsyncJobs", "EvaluationConfig", "TestCaseResults"]]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EvaluationWithViews:
        """
        ### Description

        Gets the details of a evaluation

        ### Details

        This API can be used to get information about a single evaluation by ID. To use
        this API, pass in the `id` that was returned from your Create Evaluation API
        call as a path parameter.

        Review the response schema to see the fields that will be returned.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        return await self._get(
            f"/v4/evaluations/{evaluation_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"view": view}, evaluation_retrieve_params.EvaluationRetrieveParams),
            ),
            cast_to=EvaluationWithViews,
        )

    @overload
    async def update(
        self,
        evaluation_id: str,
        *,
        annotation_config: evaluation_update_params.PartialPatchEvaluationRequestAnnotationConfig
        | NotGiven = NOT_GIVEN,
        application_spec_id: str | NotGiven = NOT_GIVEN,
        application_variant_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        evaluation_config: object | NotGiven = NOT_GIVEN,
        evaluation_config_id: str | NotGiven = NOT_GIVEN,
        evaluation_type: Literal["llm_benchmark"] | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        question_id_to_annotation_config: Dict[str, AnnotationConfigParam] | NotGiven = NOT_GIVEN,
        restore: Literal[False] | NotGiven = NOT_GIVEN,
        tags: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Evaluation:
        """
        ### Description

        Updates a evaluation

        ### Details

        This API can be used to update the evaluation that matches the ID that was
        passed in as a path parameter. To use this API, pass in the `id` that was
        returned from your Create Evaluation API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          annotation_config: Annotation configuration for tasking

          evaluation_config_id: The ID of the associated evaluation config.

          evaluation_type: If llm_benchmark is provided, the evaluation will be updated to a hybrid
              evaluation. No-op on existing hybrid evaluations, and not available for studio
              evaluations.

          question_id_to_annotation_config: Specifies the annotation configuration to use for specific questions.

          restore: Set to true to restore the entity from the database.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def update(
        self,
        evaluation_id: str,
        *,
        restore: Literal[True],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Evaluation:
        """
        ### Description

        Updates a evaluation

        ### Details

        This API can be used to update the evaluation that matches the ID that was
        passed in as a path parameter. To use this API, pass in the `id` that was
        returned from your Create Evaluation API call as a path parameter.

        Review the request schema to see the fields that can be updated.

        Args:
          restore: Set to true to restore the entity from the database.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    async def update(
        self,
        evaluation_id: str,
        *,
        annotation_config: evaluation_update_params.PartialPatchEvaluationRequestAnnotationConfig
        | NotGiven = NOT_GIVEN,
        application_spec_id: str | NotGiven = NOT_GIVEN,
        application_variant_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        evaluation_config: object | NotGiven = NOT_GIVEN,
        evaluation_config_id: str | NotGiven = NOT_GIVEN,
        evaluation_type: Literal["llm_benchmark"] | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        question_id_to_annotation_config: Dict[str, AnnotationConfigParam] | NotGiven = NOT_GIVEN,
        restore: Literal[False] | Literal[True] | NotGiven = NOT_GIVEN,
        tags: object | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Evaluation:
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        return await self._patch(
            f"/v4/evaluations/{evaluation_id}",
            body=await async_maybe_transform(
                {
                    "annotation_config": annotation_config,
                    "application_spec_id": application_spec_id,
                    "application_variant_id": application_variant_id,
                    "description": description,
                    "evaluation_config": evaluation_config,
                    "evaluation_config_id": evaluation_config_id,
                    "evaluation_type": evaluation_type,
                    "name": name,
                    "question_id_to_annotation_config": question_id_to_annotation_config,
                    "restore": restore,
                    "tags": tags,
                },
                evaluation_update_params.EvaluationUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Evaluation,
        )

    def list(
        self,
        *,
        account_id: str | NotGiven = NOT_GIVEN,
        application_spec_id: Union[int, str] | NotGiven = NOT_GIVEN,
        include_archived: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        sort_by: List[
            Literal[
                "status:asc",
                "status:desc",
                "application_spec_id:asc",
                "application_spec_id:desc",
                "application_spec:asc",
                "application_spec:desc",
                "application_variant_id:asc",
                "application_variant_id:desc",
                "application_variant:asc",
                "application_variant:desc",
                "evaluation_config_id:asc",
                "evaluation_config_id:desc",
                "completed_at:asc",
                "completed_at:desc",
                "total_test_case_result_count:asc",
                "total_test_case_result_count:desc",
                "completed_test_case_result_count:asc",
                "completed_test_case_result_count:desc",
                "annotation_config:asc",
                "annotation_config:desc",
                "question_id_to_annotation_config:asc",
                "question_id_to_annotation_config:desc",
                "evaluation_config_expanded:asc",
                "evaluation_config_expanded:desc",
                "test_case_results:asc",
                "test_case_results:desc",
                "async_jobs:asc",
                "async_jobs:desc",
                "id:asc",
                "id:desc",
                "created_at:asc",
                "created_at:desc",
                "account_id:asc",
                "account_id:desc",
                "created_by_user_id:asc",
                "created_by_user_id:desc",
                "archived_at:asc",
                "archived_at:desc",
                "created_by_user:asc",
                "created_by_user:desc",
                "name:asc",
                "name:desc",
                "description:asc",
                "description:desc",
                "tags:asc",
                "tags:desc",
                "evaluation_config:asc",
                "evaluation_config:desc",
            ]
        ]
        | NotGiven = NOT_GIVEN,
        view: List[Literal["ApplicationSpec", "AsyncJobs", "EvaluationConfig", "TestCaseResults"]]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[EvaluationWithViews, AsyncPageResponse[EvaluationWithViews]]:
        """
        ### Description

        Lists all evaluations accessible to the user.

        ### Details

        This API can be used to list evaluations. If a user has access to multiple
        accounts, all evaluations from all accounts the user is associated with will be
        returned.

        Args:
          limit: Maximum number of artifacts to be returned by the given endpoint. Defaults to
              100 and cannot be greater than 10k.

          page: Page number for pagination to be returned by the given endpoint. Starts at page
              1

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v4/evaluations",
            page=AsyncPageResponse[EvaluationWithViews],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "application_spec_id": application_spec_id,
                        "include_archived": include_archived,
                        "limit": limit,
                        "page": page,
                        "sort_by": sort_by,
                        "view": view,
                    },
                    evaluation_list_params.EvaluationListParams,
                ),
            ),
            model=EvaluationWithViews,
        )

    async def delete(
        self,
        evaluation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> GenericDeleteResponse:
        """
        ### Description

        Deletes a evaluation

        ### Details

        This API can be used to delete a evaluation by ID. To use this API, pass in the
        `id` that was returned from your Create Evaluation API call as a path parameter.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        return await self._delete(
            f"/v4/evaluations/{evaluation_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GenericDeleteResponse,
        )

    async def claim_task(
        self,
        evaluation_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Task:
        """
        Claim Evaluation Task

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not evaluation_id:
            raise ValueError(f"Expected a non-empty value for `evaluation_id` but received {evaluation_id!r}")
        return await self._post(
            f"/v4/evaluations/{evaluation_id}/claim-task",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Task,
        )


class EvaluationsResourceWithRawResponse:
    def __init__(self, evaluations: EvaluationsResource) -> None:
        self._evaluations = evaluations

        self.create = to_raw_response_wrapper(
            evaluations.create,
        )
        self.retrieve = to_raw_response_wrapper(
            evaluations.retrieve,
        )
        self.update = to_raw_response_wrapper(
            evaluations.update,
        )
        self.list = to_raw_response_wrapper(
            evaluations.list,
        )
        self.delete = to_raw_response_wrapper(
            evaluations.delete,
        )
        self.claim_task = to_raw_response_wrapper(
            evaluations.claim_task,
        )

    @cached_property
    def tasks(self) -> TasksResourceWithRawResponse:
        return TasksResourceWithRawResponse(self._evaluations.tasks)

    @cached_property
    def contributor_metrics(self) -> ContributorMetricsResourceWithRawResponse:
        return ContributorMetricsResourceWithRawResponse(self._evaluations.contributor_metrics)

    @cached_property
    def evaluation_metrics(self) -> EvaluationMetricsResourceWithRawResponse:
        return EvaluationMetricsResourceWithRawResponse(self._evaluations.evaluation_metrics)

    @cached_property
    def hybrid_eval_metrics(self) -> HybridEvalMetricsResourceWithRawResponse:
        return HybridEvalMetricsResourceWithRawResponse(self._evaluations.hybrid_eval_metrics)

    @cached_property
    def test_case_results(self) -> TestCaseResultsResourceWithRawResponse:
        return TestCaseResultsResourceWithRawResponse(self._evaluations.test_case_results)


class AsyncEvaluationsResourceWithRawResponse:
    def __init__(self, evaluations: AsyncEvaluationsResource) -> None:
        self._evaluations = evaluations

        self.create = async_to_raw_response_wrapper(
            evaluations.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            evaluations.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            evaluations.update,
        )
        self.list = async_to_raw_response_wrapper(
            evaluations.list,
        )
        self.delete = async_to_raw_response_wrapper(
            evaluations.delete,
        )
        self.claim_task = async_to_raw_response_wrapper(
            evaluations.claim_task,
        )

    @cached_property
    def tasks(self) -> AsyncTasksResourceWithRawResponse:
        return AsyncTasksResourceWithRawResponse(self._evaluations.tasks)

    @cached_property
    def contributor_metrics(self) -> AsyncContributorMetricsResourceWithRawResponse:
        return AsyncContributorMetricsResourceWithRawResponse(self._evaluations.contributor_metrics)

    @cached_property
    def evaluation_metrics(self) -> AsyncEvaluationMetricsResourceWithRawResponse:
        return AsyncEvaluationMetricsResourceWithRawResponse(self._evaluations.evaluation_metrics)

    @cached_property
    def hybrid_eval_metrics(self) -> AsyncHybridEvalMetricsResourceWithRawResponse:
        return AsyncHybridEvalMetricsResourceWithRawResponse(self._evaluations.hybrid_eval_metrics)

    @cached_property
    def test_case_results(self) -> AsyncTestCaseResultsResourceWithRawResponse:
        return AsyncTestCaseResultsResourceWithRawResponse(self._evaluations.test_case_results)


class EvaluationsResourceWithStreamingResponse:
    def __init__(self, evaluations: EvaluationsResource) -> None:
        self._evaluations = evaluations

        self.create = to_streamed_response_wrapper(
            evaluations.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            evaluations.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            evaluations.update,
        )
        self.list = to_streamed_response_wrapper(
            evaluations.list,
        )
        self.delete = to_streamed_response_wrapper(
            evaluations.delete,
        )
        self.claim_task = to_streamed_response_wrapper(
            evaluations.claim_task,
        )

    @cached_property
    def tasks(self) -> TasksResourceWithStreamingResponse:
        return TasksResourceWithStreamingResponse(self._evaluations.tasks)

    @cached_property
    def contributor_metrics(self) -> ContributorMetricsResourceWithStreamingResponse:
        return ContributorMetricsResourceWithStreamingResponse(self._evaluations.contributor_metrics)

    @cached_property
    def evaluation_metrics(self) -> EvaluationMetricsResourceWithStreamingResponse:
        return EvaluationMetricsResourceWithStreamingResponse(self._evaluations.evaluation_metrics)

    @cached_property
    def hybrid_eval_metrics(self) -> HybridEvalMetricsResourceWithStreamingResponse:
        return HybridEvalMetricsResourceWithStreamingResponse(self._evaluations.hybrid_eval_metrics)

    @cached_property
    def test_case_results(self) -> TestCaseResultsResourceWithStreamingResponse:
        return TestCaseResultsResourceWithStreamingResponse(self._evaluations.test_case_results)


class AsyncEvaluationsResourceWithStreamingResponse:
    def __init__(self, evaluations: AsyncEvaluationsResource) -> None:
        self._evaluations = evaluations

        self.create = async_to_streamed_response_wrapper(
            evaluations.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            evaluations.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            evaluations.update,
        )
        self.list = async_to_streamed_response_wrapper(
            evaluations.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            evaluations.delete,
        )
        self.claim_task = async_to_streamed_response_wrapper(
            evaluations.claim_task,
        )

    @cached_property
    def tasks(self) -> AsyncTasksResourceWithStreamingResponse:
        return AsyncTasksResourceWithStreamingResponse(self._evaluations.tasks)

    @cached_property
    def contributor_metrics(self) -> AsyncContributorMetricsResourceWithStreamingResponse:
        return AsyncContributorMetricsResourceWithStreamingResponse(self._evaluations.contributor_metrics)

    @cached_property
    def evaluation_metrics(self) -> AsyncEvaluationMetricsResourceWithStreamingResponse:
        return AsyncEvaluationMetricsResourceWithStreamingResponse(self._evaluations.evaluation_metrics)

    @cached_property
    def hybrid_eval_metrics(self) -> AsyncHybridEvalMetricsResourceWithStreamingResponse:
        return AsyncHybridEvalMetricsResourceWithStreamingResponse(self._evaluations.hybrid_eval_metrics)

    @cached_property
    def test_case_results(self) -> AsyncTestCaseResultsResourceWithStreamingResponse:
        return AsyncTestCaseResultsResourceWithStreamingResponse(self._evaluations.test_case_results)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from .._models import BaseModel

__all__ = ["StudioProject"]


class StudioProject(BaseModel):
    id: str
    """The unique identifier of the entity."""

    account_id: str
    """The ID of the account that owns the given entity."""

    created_at: datetime
    """The date and time when the entity was created in ISO format."""

    created_by_user_id: str
    """The user who originally created the entity."""

    description: str
    """The description of the Studio Project"""

    name: str
    """The name of the Studio Project"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypeAlias, TypedDict

from ..._utils import PropertyInfo
from ..token_chunking_strategy_config_param import TokenChunkingStrategyConfigParam
from ..custom_chunking_strategy_config_param import CustomChunkingStrategyConfigParam
from ..character_chunking_strategy_config_param import CharacterChunkingStrategyConfigParam

__all__ = ["UploadScheduleCreateParams", "ChunkingStrategyConfig"]


class UploadScheduleCreateParams(TypedDict, total=False):
    chunking_strategy_config: Required[ChunkingStrategyConfig]

    interval: Required[float]

    knowledge_base_data_source_id: Required[str]

    account_id: str
    """The ID of the account that owns the given entity."""

    next_run_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]


ChunkingStrategyConfig: TypeAlias = Union[
    CharacterChunkingStrategyConfigParam, TokenChunkingStrategyConfigParam, CustomChunkingStrategyConfigParam
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Required, TypeAlias, TypedDict

from .s3_data_source_config_param import S3DataSourceConfigParam
from .slack_data_source_config_param import SlackDataSourceConfigParam
from .s3_data_source_auth_config_param import S3DataSourceAuthConfigParam
from .confluence_data_source_config_param import ConfluenceDataSourceConfigParam
from .slack_data_source_auth_config_param import SlackDataSourceAuthConfigParam
from .share_point_data_source_config_param import SharePointDataSourceConfigParam
from .google_drive_data_source_config_param import GoogleDriveDataSourceConfigParam
from .confluence_data_source_auth_config_param import ConfluenceDataSourceAuthConfigParam
from .share_point_data_source_auth_config_param import SharePointDataSourceAuthConfigParam
from .google_drive_data_source_auth_config_param import GoogleDriveDataSourceAuthConfigParam
from .azure_blob_storage_data_source_config_param import AzureBlobStorageDataSourceConfigParam
from .azure_blob_storage_data_source_auth_config_param import AzureBlobStorageDataSourceAuthConfigParam

__all__ = ["KnowledgeBaseDataSourceCreateParams", "DataSourceConfig", "DataSourceAuthConfig"]


class KnowledgeBaseDataSourceCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    data_source_config: Required[DataSourceConfig]

    name: Required[str]

    data_source_auth_config: DataSourceAuthConfig

    description: str


DataSourceConfig: TypeAlias = Union[
    S3DataSourceConfigParam,
    SharePointDataSourceConfigParam,
    GoogleDriveDataSourceConfigParam,
    AzureBlobStorageDataSourceConfigParam,
    ConfluenceDataSourceConfigParam,
    SlackDataSourceConfigParam,
]

DataSourceAuthConfig: TypeAlias = Union[
    SharePointDataSourceAuthConfigParam,
    AzureBlobStorageDataSourceAuthConfigParam,
    GoogleDriveDataSourceAuthConfigParam,
    S3DataSourceAuthConfigParam,
    ConfluenceDataSourceAuthConfigParam,
    SlackDataSourceAuthConfigParam,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .scalar import (
    ScalarResource,
    AsyncScalarResource,
    ScalarResourceWithRawResponse,
    AsyncScalarResourceWithRawResponse,
    ScalarResourceWithStreamingResponse,
    AsyncScalarResourceWithStreamingResponse,
)
from ...._compat import cached_property
from .timeseries import (
    TimeseriesResource,
    AsyncTimeseriesResource,
    TimeseriesResourceWithRawResponse,
    AsyncTimeseriesResourceWithRawResponse,
    TimeseriesResourceWithStreamingResponse,
    AsyncTimeseriesResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["MetricsResource", "AsyncMetricsResource"]


class MetricsResource(SyncAPIResource):
    @cached_property
    def scalar(self) -> ScalarResource:
        return ScalarResource(self._client)

    @cached_property
    def timeseries(self) -> TimeseriesResource:
        return TimeseriesResource(self._client)

    @cached_property
    def with_raw_response(self) -> MetricsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return MetricsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MetricsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return MetricsResourceWithStreamingResponse(self)


class AsyncMetricsResource(AsyncAPIResource):
    @cached_property
    def scalar(self) -> AsyncScalarResource:
        return AsyncScalarResource(self._client)

    @cached_property
    def timeseries(self) -> AsyncTimeseriesResource:
        return AsyncTimeseriesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncMetricsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMetricsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMetricsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncMetricsResourceWithStreamingResponse(self)


class MetricsResourceWithRawResponse:
    def __init__(self, metrics: MetricsResource) -> None:
        self._metrics = metrics

    @cached_property
    def scalar(self) -> ScalarResourceWithRawResponse:
        return ScalarResourceWithRawResponse(self._metrics.scalar)

    @cached_property
    def timeseries(self) -> TimeseriesResourceWithRawResponse:
        return TimeseriesResourceWithRawResponse(self._metrics.timeseries)


class AsyncMetricsResourceWithRawResponse:
    def __init__(self, metrics: AsyncMetricsResource) -> None:
        self._metrics = metrics

    @cached_property
    def scalar(self) -> AsyncScalarResourceWithRawResponse:
        return AsyncScalarResourceWithRawResponse(self._metrics.scalar)

    @cached_property
    def timeseries(self) -> AsyncTimeseriesResourceWithRawResponse:
        return AsyncTimeseriesResourceWithRawResponse(self._metrics.timeseries)


class MetricsResourceWithStreamingResponse:
    def __init__(self, metrics: MetricsResource) -> None:
        self._metrics = metrics

    @cached_property
    def scalar(self) -> ScalarResourceWithStreamingResponse:
        return ScalarResourceWithStreamingResponse(self._metrics.scalar)

    @cached_property
    def timeseries(self) -> TimeseriesResourceWithStreamingResponse:
        return TimeseriesResourceWithStreamingResponse(self._metrics.timeseries)


class AsyncMetricsResourceWithStreamingResponse:
    def __init__(self, metrics: AsyncMetricsResource) -> None:
        self._metrics = metrics

    @cached_property
    def scalar(self) -> AsyncScalarResourceWithStreamingResponse:
        return AsyncScalarResourceWithStreamingResponse(self._metrics.scalar)

    @cached_property
    def timeseries(self) -> AsyncTimeseriesResourceWithStreamingResponse:
        return AsyncTimeseriesResourceWithStreamingResponse(self._metrics.timeseries)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

__all__ = ["UploadScheduleRetrieveParams"]


class UploadScheduleRetrieveParams(TypedDict, total=False):
    knowledge_base_id: Required[str]

    view: List[Literal["DataSource"]]
